# File tres/Scripts/ctres.cl
# February 13, 2008
# By Doug Mink, SAO Telescope Data Center

# CTRES - Process TRES images from raw data files, removing cosmic rays, to spectra

procedure ctres (images)

string	images=""	{prompt="Images to process"}
string	datedir=""	{prompt="Date for images"}
string	instdir=""	{prompt="Directory containing date directory"}
bool	fixpreamp=yes	{prompt="Run tpreamp to fix time constant (yes or no)"}
bool	fixpix=yes	{prompt="Fix bad pixels (yes or no)"}
bool	fixbias=no	{prompt="Fix bias section (yes or no)"}
real	gaincorr=1.0	{prompt="Gain correction for amp2 to amp1"}
bool	flatten=yes	{prompt="Run TFLAT to Flatten image (yes or no)"}
string	normfile=""	{prompt="Pixel to pixel tmakeflat normalization file"}
string	normdir="tresdata$"	{prompt="Directory for normalization file"}
string	aproot="flat"	{prompt="Root file name for aperture reference file"}
file	apdir="tresdata$" {prompt="Directory for aperture reference file"}
bool	recenter=no	{prompt="Recenter orders during extraction (yes or no)"}
bool	disperse=yes	{prompt="Dispersion correct extracted spectra (yes or no)"}
string	comproot="comp"	{prompt="Root file name for calibration lamp file"}
file	compdir="tresdata$"	{prompt="Directory for calibration lamp file"}
int	plotorder=1	{prompt="Echelle order to plot (No plot if zero"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	saveraw=yes	{prompt="Save unprocessed image files (y or n)"}
bool	saveold=yes	{prompt="Save intermediate image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

begin

file	imroot, imin, impath
file	imout, ims, cdir, tempin, tempout
string	ddir, idir, croot
int	pl, n
bool	vb, dt, sr, sv, disp, rc

	vb = verbose
	dt = deltemp
	sv = saveold
	sr = saveraw
	rc = recenter
	pl = plotorder
	ddir = datedir
	idir = instdir
	ims = images
	cdir = compdir
	croot = comproot
	disp = disperse

	if (ims == "") {
	    print ("CTRES: Reduce TRES images in a different directory")
	    print ("Usage: ctres images with both amps to be corrected")
	    return
	    }

#  Make temporary list of input files
	tempin = mktemp ("tempctimg") // ".list"
	files (ims, >tempin)
	tempout = mktemp ("tempctfib") // ".list"
	if (access (tempout))
	    delete (tempout)

# Figure out file names and copy images to current directory
	if (instdir != "") {
	    fixdir (idir)
	    list = tempin
	    while (fscan (list,imin) != EOF) {

# If a number is given, prepend appropriate zeroes and wildcard
		isnum (imin)
		if (isnum.fpcode > 0) {
		    n = int (imin)
		    if (n < 10)
			imin = "000" // n // ".*"
		    else if (n < 100)
			imin = "00" // n // ".*"
		    else if (n < 1000)
			imin = "0" // n // ".*"
		    else
			imin = "0" // n // ".*"
		    }

# If file extension is missing, add wildcard
		else {
		    smatch (imin, ".fits")
		    if (smatch.found == no)
			imin = imin // "*"
		    }
		impath = fixdir.newdir
		if (ddir != "")
		    impath = impath // ddir // "/" // imin
		else
		    impath = impath // imin
#		if (vb)
#		    print ("CTRES: Copying file ",impath)
		filepath (impath)
		if (!access (filepath.files)) {
		    print ("CTRES: Cannot read file ",filepath.files)
		    }
		filename (filepath.files)
		if (ddir != "")
		    imout = ddir // "." // filename.filename
		else
		    imout = filename.filename
		if (!access (imout)) {
		    if (vb)
			print ("CTRES: Copying file ",filepath.files," to ", imout)
		    copy (impath, imout)
		    }
		else if (vb)
		    print ("CTRES: File ", imout, " already exists")
		print (imout, >>tempout)
		}
	    }

	ims = "@" // tempout

# Remove overscan, trim, and merge amplifiers of image
# Then remove pixel to pixel variation from image (including IR fringing)
# Then extract spectra from image and add dispersion function
	tproc.gaincorr = gaincorr
	tproc.fixbias = fixbias
	tproc.fixpix = fixpix
	tproc.fixpreamp = fixpreamp
	tproc.flatten = flatten
	tproc.extract = yes
	tproc.disperse = disperse
	tproc.normfile = normfile
	tproc.normdir = normdir
	tproc.aproot = aproot
	tproc.apdir = apdir
	tproc.refroot = croot
	tproc.refdir = cdir
	tproc.recenter = rc
	tproc.deltemp = dt
	tproc.saveold = sv
	tproc.verbose = vb
	tproc (ims)

# Plot extracted spectra if desired
	if (pl > 0) {
	    list = tempout
	    while (fscan (list,imout) != EOF) {
		fileroot (imout)
		imroot = fileroot.rootname
		ims = imroot // "_fib1.ec.fits"
		if (access (ims)) {
		    splot (ims,pl)
		    }
		ims = imroot // "_fib2.ec.fits"
		if (access (ims)) {
		    splot (ims,pl)
		    }
		ims = imroot // "_fib3.ec.fits"
		if (access (ims)) {
		    splot (ims,pl)
		    }
		}
	    }

#  Delete temporary list files
	if (dt) {
	    delete (tempin)
	    delete (tempout)
	    }

end

# Jan 17 2008	New task for quick look with cosmic ray removal
# Jan 25 2008	Access data by sequence number as well as file name
# Jan 25 2008	Add wavelength calibration and plotting
# Jan 30 2008	Use tdisp to add wavelength solution
# Jan 30 2008	Add parameters apdir and aproot for extraction templates
# Feb 11 2008	Change temporary file names from ctresin and ctresout
# Feb 12 2008	Use TPROC to process spectra
# Feb 13 2008	Change parameter imdate to datedir
