# TPREAMP - Run SPP version of Brian McCleod's preamplifier fix on raw images
# February 11, 2008
# By Doug Mink, SAO Telescope Data Center

procedure tpreamp (images)

string	images=""	{prompt="Images to process"}

string	imdir=""	{prompt="Directory for image files"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	saveold=yes	{prompt="Save unprocessed image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (yes or no)"}
bool	debug=no	{prompt="Print more process progress (yes or no)"}

struct *preampfiles
begin

file	imroot, temp1, imin, templist, imout, rawdir, imd, impath
string	imx, ims, imsec, history, tchar
string	pfsect0, pfsect1, pfsect2
bool	vb, dt, db, sv

	vb = verbose
	dt = deltemp
	db = debug
	imd = imdir
	imx = ".fits"
	ims = image
	sv = saveold

	if (ims == "") {
	    print ("TPREAMP: Correct preamp problem for all four amps one amp at a time")
	    print ("Usage: hpreamp <input images with 4 extensions>")
	    return
	    }

# Make directory in which to save raw multi-extension files
	if (sv) {
	    makepath ("Unpreamp", imd)
	    rawdir = makepath.filepath
	    if (!access (rawdir))
		mkdir (rawdir)
	    }

#  Make temporary list of input files
	tchar = substr (ims,1,1)
	if (tchar == '@') {
	    templist = substr (ims,2,100)
	    deltemp = no
	    }
	else {
	    templist = mktemp ("tempamp") // ".list"
	    files (ims, >templist)
	    }
	preampfiles = templist
        while (fscan (preampfiles, imin) != EOF) {

# Do not process images without extensions
	    imgetp.default = "-1"
	    imsec = imin // "[0]"
	    imgetp (imsec, "i_naxis")
	    if (imgetp.value != "0") {
		print ("TPREAMP: ", imin, " is not a multi-extension image")
		print ("NAXIS = ", imgetp.value)
		next
		}

# Do not process image twice
	    imgetp.default = "0"
	    imsec = imin // "[1]"
	    imgetp (imsec, "PREAMPFX")
	    if (imgetp.value != "0") {
		if (vb) {
		    print ("TPREAMP: ", imin, " has already been processed")
		    print ("PREAMPFX = ", imgetp.value)
		    }
		next
		}

# Find root name of image
	    imroot = imin
	    smatch (imroot, imx)
	    if (smatch.found) {
		imroot = substr (imroot,1,smatch.start-1)
		}
	    else {
		imroot = imin
		imin = imin // imx
		}

# Fix preamp output using SPP version of Brian McLeod's C program preampfix
	    if (db) {
		print ("TPREAMP: Making preamp correction for ",imin)
		print ("TPREAMP: image root is ",imroot)
		}
# Output filename is the same as the inpugreated
	    tampfix.verbose = vb
	    tampfix.debug = db
	    tampfix.error = ""
	    tampfix.indir = imd
	    pfsect0 = imroot // "_0.fits"
	    imcopy (imin // "[0]", pfsect0, verbose=vb)
	    pfsect1 = imroot // "_1.fits"
	    imcopy (imin // "[1]", pfsect1, verbose=vb)
	    tampfix (pfsect1)
	    if (tampfix.error != "")
		print ("TAMPFIX 1: ",tampfix.error)
	    pfsect2 = imroot // "_2.fits"
	    imcopy (imin // "[2]", pfsect2, verbose=vb)
	    tampfix.error = ""
	    tampfix (pfsect2)
	    if (tampfix.error != "")
		print ("TAMPFIX 2: ",tampfix.error)

# Save unchanged image
	    makepath (imin, imd)
	    impath = makepath.filepath
	    if (sv) {
		imout = rawdir // "/" // imin
		if (!access (imout))
		    rename (impath, imout)
		else
		    delete (impath)
		}
	    else
		delete (impath)

# Reassemble the multi-extension FITS file
	    if(vb)
		print("TPREAMP: rejoining ",imin)

# Reassemble the input file with the fixed extensions
	    mscjoin (imroot, output = impath, delete=dt, verbose=vb)
	    }

	if (deltemp)
	    delete (templist)
	preampfiles = ""
end

# Jan 10 2008	New task based on hectospec.hpreamp
# Feb  8 2008	Save unprocessed images in Unpreamp/, not Raw/
# Feb 11 2008	Change name of temporary file from proclist* to tempamp*list
