# TPROC - Run image processing tasks on both TRES amps, combine, and flatten
# February 12, 2008
# By Doug Mink, SAO Telescope Data Center

procedure tproc (images)

string	images=""	{prompt="Images to merge"}
real	gaincorr=1.0	{prompt="Gain correction for amp2 to amp1"}
bool	fixpreamp=yes	{prompt="Make preamp correction (yes or no)"}
bool	fixbias=no	{prompt="Fix bias section (yes or no)"}
bool	fixpix=yes	{prompt="Fix bad pixels (yes or no)"}
bool	flatten=yes	{prompt="Run tflat on each image (yes or no)"}
bool	cosmic=yes	{prompt="Remove cosmic rays from a list of images (yes or no)"}
bool	extract=yes	{prompt="Extract spectra using textract (yes or no)"}
bool	disperse=yes	{prompt="Add wavelength dispersion function to spectrum (yes or no)"}
string  normfile=""     {prompt="Pixel to pixel tmakeflat normalization file"}
string  normdir="tresdata$"     {prompt="Directory for normalization file"}
string	aproot="flat"	{prompt="Root filename for aperature reference spectrum"}
string	apdir="tresdata$"	{prompt="Directory for reference spectrum"}
bool	recenter=no	{prompt="Recenter orders during extraction (yes or no)"}
string	refroot="comp"	{prompt="Root filename for wavelength reference spectrum"}
string	refdir="tresdata$"	{prompt="Directory for reference spectrum"}
real	refshift=0.	{prompt="Pixel shift from reference comparison spectrum"}
bool	deltemp=yes	{prompt="Delete intermediate image lists (yes or no)"}
bool	saveraw=yes	{prompt="Save unprocessed image files (yes or no)"}
bool	saveold=yes	{prompt="Save intermediate unprocessed image files (yes or no)"}
bool	verbose=yes	{prompt="Print process progress (yes or no)"}

struct *procfiles
begin

file	imroot, amp1, amp2, temp1, temp2, temp3, im2, imin, templist
file	imfile, newfile, imlist
string	chip1, chip2, arg1, imx, ims, imsec, history, tchar, imex
string	binning, nd, nf, temparg, apd, apr
bool	vb, dt, sv, fp, fb, flat, disp, recent
int	nfiles
real	gc

	vb = verbose
	dt = deltemp
	sv = saveold
	fp = fixpreamp
	fb = fixbias
	flat = flatten
	recent = recenter
	disp = disperse
	gc = gaincorr
	nf = normfile
	nd = normdir
	apr = aproot
	apd = apdir
	imx = ".fits"
	ims = image

	if (ims == "") {
	    print ("TPROC: Process TRES image files to extraction")
	    print ("Usage: tproc im1,im2,...,imn")
	    print ("Usage: tproc *.fits")
	    print ("Usage: tproc @image_list_file")
	    return
	    }

	if (!defpac ("mscred"))
	    mscred

#  Make temporary list of input files
	tchar = substr (ims,1,1)
	templist = mktemp ("tempproc") // ".list"
	if (tchar == '@') {
	    imlist = substr (ims,2,100)
	    copy (imlist, templist)
	    }
	else {
	    files (ims, >templist)
	    }
	tpreproc.fixpix = fixpix

# Read through input files
	procfiles = templist
	nfiles = 0
        while (fscan (procfiles,imin) != EOF) {

# Process files as images
	    tpreproc (imin,gaincorr=gc,fixpreamp=fp,fixbias=fb,deltemp=dt,saveold=sv,verbose=vb)

# Flatfield correct spectra
	    if (flat)
		tflat (imin,normfile=nf,normdir=nd,deltemp=dt,saveold=sv,verbose=vb)
	    nfiles = nfiles + 1
	    }

# Remove cosmic rays, if requested
	if (cosmic) {
	    if (nfiles > 1) {
		temparg = "@" // templist
		tcosmic (temparg)
		}
	    else if (vb) {
		print ("TPROC: Cannot remove cosmic rays from a single file")
		}
	    }

# Extract 1D x orders spectrum and add dispersion, if requested
	if (extract) {
	    procfiles = templist
            while (fscan (procfiles,imin) != EOF) {
		textract (imin, aproot=apr, apdir=apd, recenter=recent)
		if (disp) {
		    if (textract.spec1 != "")
			tdisp (textract.spec1)
		    if (textract.spec2 != "")
			tdisp (textract.spec2)
		    if (textract.spec3 != "")
			tdisp (textract.spec3)
		    }
		}
	    }

	if (dt)
	    delete (templist)
	procfiles = ""
end

# Jan 09 2008	New program to process list of TRES images
# Jan 17 2008	Update normfile and normdir prompts
# Jan 24 2008	Add optional bad pixel file correction with fixpix parameter
# Jan 28 2008	Add optional cosmic ray removal from list of images
# Jan 31 2008	Add optional spectrum extraction from list of images
# Feb  8 2008	Add saveraw to save raw images but not others
# Feb  8 2008	Add optional wavelength dispersion
# Feb 11 2008	Change temporary list file from temproclist* to tempproc*list
# Feb 12 2008	Add recenter parameter for extraction
