# File tres/Scripts/trslist.cl
# By Doug Mink
# February 13, 2008

# TRSLIST -- Make Starbase table of major keywords in TRES data files
#            and make lists for FLATs and COMPs

procedure trslist (images)

string images="1-300"	{prompt="Files to process"}

string datedir=""	{prompt="Date for images"}
string instdir=""	{prompt="Directory containing date directories"}
string imagetype=""	{prompt="Select this image type if not null"}
string object=""	{prompt="Select this object if not null"}
string apfib=""		{prompt="Select fiber (s m l) binning (b) ap (1 2 3)"}
string fibkey=""	{prompt="Select on this FIBKEY value"}
string binning=""	{prompt="Select if binned this many times (1 2 null)"}
string fibsize=""	{prompt="Select if FIBSIZ is this value"}
string exptime=""	{prompt="Select if EXPTIME has this value"}
file outlist=""		{prompt="File name for output list of files"}
bool addfib=yes		{prompt="Add FIBi keywords to spectra (yes or no)"}
bool deltemp=yes	{prompt="Delete temporary files (yes or no"}
bool verbose=yes	{prompt="Print table of image parameters (yes or no)"}
bool debug=no		{prompt="Print table with more image parameters (yes or no)"}

begin
    bool vb, db, dt
    file imf, tempin, tempout, idir, imout, imdir
    file listfile, ofile, impath, imin
    string fkey, fib1, fib2, fib3, af, aff, afb, afs, afx
    string bin1, ims, ddir, fk, fsize, fs, fb, imt
    int ns, n, nrange, lfile, maxlfile, lobj, maxlobj
    string specs, olist, lastpath, obj
    string fname, fpath
    string dline1, dline2
    char tchar

    vb = verbose
    db = debug
    dt = deltemp
    ims = images
    idir = instdir
    ddir = datedir
    ofile = outlist
    af = apfib
    afx = af
    fk = fibkey
    fs = fibsize
    fb = binning
    imt = imagetype

# Specify fiber size
    smatch (afx, "s")
    if (smatch.found)
	afs = "s"
    else {
	smatch (afx, "m")
	if (smatch.found)
	    afs = "m"
	else {
	    smatch (afx, "l")
	    if (smatch.found)
		afs = "l"
	    else
		afs = ""
	    }
	}

# Specify binning
    smatch (afx, "b")
    if (smatch.found)
	afb = "b"
    else
	afb = ""

# Specify fiber number
    smatch (afx, "1")
    if (smatch.found)
	aff = "1"
    else {
	smatch (afx, "2")
	if (smatch.found)
	    aff = "2"
	else {
	    smatch (afx, "3")
	    if (smatch.found)
		aff = "3"
	    else
		aff = ""
	    }
	}

#  Make temporary list of input files
    tempin = mktemp ("templistin")
    if (access (tempin))
	delete (tempin)
    isrange (ims)
    if (isrange.range) {
	getrange (ims, 0)
	nrange = getrange.nval
	for (i = 1; i <= nrange; i = i + 1) {
	    getrange (ims, i)
	    if (i == 1)
		print (getrange.ival, > tempin)
	    else
		print (getrange.ival, >> tempin)
	    }
	}
    else
	files (ims, >tempin)
    tempout = mktemp ("templistout")
    if (access (tempout))
	delete (tempout)
    maxlfile = 0
    maxlobj = 0

# Make list of image paths if instrument directory is specified
    if (idir != "" || isrange.range) {
	addfib = no
	fixdir (idir)
	imdir = fixdir.newdir
	if (ddir != "") {
	    imdir = imdir // ddir
	    fixdir (imdir)
	    imdir = fixdir.newdir
	}
	if (!access (imdir)) {
	    print ("TRSLIST cannot read directory ",imdir)
	    return
	    }
	lastpath = ""

	list = tempin
	while (fscan (list,imin) != EOF) {

# If a number is given, prepend appropriate zeroes and wildcard
	    isnum (imin)
	    if (isnum.fpcode > 0) {
		n = int (imin)
		if (n < 10)
		    imin = "000" // n // ".*"
		else if (n < 100)
		    imin = "00" // n // ".*"
		else if (n < 1000)
		    imin = "0" // n // ".*"
		else
		    imin = n // ".*"
		}

# If file extension is missing, add wildcard
	    else {
		smatch (imin, ".fits")
		if (smatch.found == no)
		    imin = imin // "*"
		}
	    impath = imdir // imin
	    filepath.files = ""
	    filepath (impath)
	    impath = filepath.files
	    if (!access (impath)) {
		print ("TRSLIST: Cannot read file ",impath)
		break
		}
	    if (impath != lastpath) {
		if (db)
		    print ("TRSLIST: Reading file ",impath)
		print (impath, >>tempout)
		}
	    lastpath = impath
	    filename (impath)
	    lfile = strlen (filename.filename)
	    if (lfile > maxlfile)
		maxlfile = lfile
	    impath = impath // "[0]"
	    imgetp (impath, "i_title")
	    lobj = strlen (imgetp.value)
	    if (lobj > maxlobj)
		maxlobj = lobj
	    }
	}
    else
	tempout = tempin
    if (maxlfile < 8)
	maxlfile = 8
    if (maxlobj < 6)
	maxlobj = 6

# Print heading for list of spectrum attributes
    if (debug) {
	fname = "filename"
	lfile = strlen (fname)
	for (i = lfile+1; i <= maxlfile; i = i + 1) {
	    fname = fname // " "
	    }
	obj = "object"
	for (i = 7; i <= maxlfile; i = i + 1) {
	    obj = obj // " "
	    }
	dline1 = fname // "    	type	" // obj // "	peri	bin	fibsize	ossw	exptime	cmptime	fibkey	fib1	fib2	fib3"
	fname = "-"
	for (i = 2; i <= maxlfile; i = i + 1) {
	    fname = fname // "-"
	    }
	obj = "-"
	for (i = 2; i <= maxlobj; i = i + 1) {
	    obj = obj // "-"
	    }
	dline2 = fname // "	-------	" // obj // "	----	---	-------	----	-------	-------	------	----	----	----"
	}
    else {
	fname = "filename"
	for (i = 9; i <= maxlfile; i = i + 1) {
	    fname = fname // " "
	    }
	obj = "object"
	for (i = 7; i <= maxlobj; i = i + 1) {
	    obj = obj // " "
	    }
	dline1 = fname // "	type	"// obj // "	bin	fibsize	fibkey	exptime	cmptime"
	fname = "-"
	for (i = 2; i <= maxlfile; i = i + 1) {
	    fname = fname // "-"
	    }
	obj = "-"
	for (i = 2; i <= maxlobj; i = i + 1) {
	    obj = obj // "-"
	    }
	dline2 = fname // "	-------	" // obj // "	----	-------	------	-------	-------"
	}
    
    trsfib.addfib = addfib
    trsfib.verbose = no
    trsfib.debug = no

    ns = 0
    if (vb) {
	print (imdir)
	print (dline1)
	print (dline2)
	}

# Print spectrum attributes
    olist = ""
    list = tempout
    while (fscan (list, imf) != EOF) {
	fpath = imf
	filename (fpath)
	fname = filename.filename
	trsfib.specdir = filename.dirpath
	trsfib (fname)
#	if (db)
#	    dpar ("trsfib")
	fkey = trsfib.fibkey
	fib1 = trsfib.fib1
	fib2 = trsfib.fib2
	fib3 = trsfib.fib3
	fsize = trsfib.fibsize
	bin1 = trsfib.bin1
	if (db)
	    print ("AF=",af," AFF=",aff," afs=",afs," afb=",afb)
	if (af != "") {
	    if (aff == "1" && fib1 == "OFF")
		next
	    if (aff == "2" && fib2 == "OFF")
		next
	    if (aff == "3" && fib3 == "OFF")
		next
	    if (afb != bin1)
		next
	    if (afs != "" && afs != trsfib.apsize)
		next
	    }
	if (imt != "" && trsfib.imagetype != imt)
	    next
	if (fk != "" && fkey != fk)
	    next
	if (fs != "" && fsize != fs)
	    next
	if (fb != "" && bin1 != fb)
	    next
	if (object != "" && trsfib.object != object)
	    next
	if (exptime != "" && trsfib.exptime != exptime)
	    next
	ns = ns + 1
	lfile = strlen (fname)
	for (i = lfile+1; i <= maxlfile; i = i + 1) {
	    fname = fname // " "
	    }
	obj = trsfib.object
	lobj = strlen (obj)
	for (i = lobj+1; i <= maxlobj; i = i + 1) {
	    obj = obj // " "
	    }

	if (db)
	    printf ("%s	%s	%s	%s	%s	%s	%s	%s	%s	%s	%s	%s	%s\n", fname, trsfib.imagetype, trsfib.object, trsfib.peri, bin1, fsize, trsfib.osswitch, trsfib.exptime, trsfib.comptime, fkey, fib1, fib2, fib3)
	else if (vb)
	    printf ("%s	%s	%s	%s	%s	%s	%s	%s\n", fname, trsfib.imagetype, obj, bin1, fsize, fkey, trsfib.exptime, trsfib.comptime)

# Save this file name in list or listfile
	if (ns == 1) {
	    if (ofile != "")
		print (fname, > ofile)
	    else
		olist = fname
	    }
	else {
	    if (ofile != "")
		print (fname, >> ofile)
	    else
		olist = olist // "," // fname
	    }
	}

    if (dt) {
	if (access (tempin))
	    delete (tempin)
	if (access (tempout))
	    delete (tempout)
	}

    if (!vb && !db && olist != "")
	print (olist)

end

# Oct  1 2007	New program
# Oct  5 2007	Use tresfib to set FIBi keyword values
# Oct  9 2007	Use header parameter values from tresfib, not gethead
# Oct 10 2007	If speckey.db already exists, print it if verbose=yes
# Oct 11 2007	Add apfib and object keywords to use treslist for sorting
# Oct 11 2007	If not verbose and conditions set, output a list of files
# Oct 11 2007	Set speckey.db in tresdb.cl, not here
# Oct 12 2007	Update parameter list
# Oct 25 2007	Add output file for long lists meeting apfib/object condition
# Dec 19 2007	Add parameter fibkey and test for equality to it if not null

# Jan 10 2008	Change name from treslist to trslist
# Jan 10 2008	Changes calls to tresfib to trsfib
# Jan 28 2008	Add optional check for a specific exposure time
# Jan 30 2008	Add imagetype to list only spectra from a specific object type
# Jan 30 2008	Parse observation sequence numbers from arbitrary directory
# Jan 31 2008	Shrink the verbose output table,leaving a bigger one for debug
# Feb 12 2008	Change parameter datedir to imdate for constency with QTRES and CTRES
# Feb 13 2008	Change parameter imdate back to datedir because it makes more sense
